@@ -32,16 +32,20 @@ class Spyglass:
 
     def __init__(self, url, username, password, dataset_id=None):
         self._url = url
-        websocket.enableTrace(False)
-        self._ws = websocket.WebSocketApp(url,
-                                          on_message=self._on_message,
-                                          on_error=self._on_error,
-                                          on_close=self._on_close)
-        self._ws.on_open = self._on_open
         self._username = username
         self._password = password
         self.dataset_id = dataset_id
+        self.connect()
+
+    def connect(self):
+        websocket.enableTrace(False)
+        self._ws = websocket.WebSocketApp(self._url,
+                                          on_open=self._on_open,
+                                          on_message=self._on_message,
+                                          on_error=self._on_error,
+                                          on_close=self._on_close)
         self._connecting = True
+        self.connected = False
         thread.start_new_thread(self._ws.run_forever, ())
         self.results = {}
         # wait for conection and auth
@@ -51,8 +55,8 @@ class Spyglass:
         if not self.connected:
             raise Exception("Could not connect")
 
-        if dataset_id:
-            self.cmd("session", dataset_id)
+        if self.dataset_id:
+            self.cmd("session", self.dataset_id)
 
     def _on_open(self):
         def _auth(*args):
@@ -81,6 +85,11 @@ class Spyglass:
             self.results[message['seqno']] = message
 
     def _on_error(self, error):
+        # if connecting stop trying
+        if self._connecting:
+            self._connecting = False
+        elif 'Errno 54' in error:
+            self.connect()
         raise Exception(error)
 
     def _on_close(self):
@@ -101,7 +110,6 @@ class Spyglass:
         return build_command
 
     def cmd(self, command, *args, **kwargs):
-
         def _get_session():
             if not self.dataset_id:
                 raise Exception('Need a dataset_id to get a session')
@@ -123,12 +131,21 @@ class Spyglass:
 
         logging.info(
             'sending sync command: {} {}'.format(seqno, final_command))
-        # if lost connection try to reconnect then re-execute command
         try:
             self._ws.send("{} {}".format(seqno, final_command))
+            # wait for result
             while self.seqno not in self.results:
+                # if we loose conenction whiile waiting for result we exit
+                if self._ws.sock is None:
+                    logger.warn("we lost connection")
+                    raise websocket.WebSocketConnectionClosedException()
                 sleep(0.1)
             result = self.results[self.seqno]
+        except websocket.WebSocketConnectionClosedException as e:
+            logger.warn("Connection Closed")
+            self.connect()
+            kwargs['retry'] = retry + 1
+            return self.cmd(command, *args, **kwargs)
         except Exception as e:
             logger.warn(e)
             kwargs['retry'] = retry + 1
